
#ifdef DYRKI_WIN32
#include <io.h>		// _getcwd
#elif defined DYRKI_LINUX
#include <unistd.h>	// getcwd
#endif

#include <assert.h>

#include <vector>
#include <iostream>
using namespace std;

#include "mmgr.h"

#include "application.h"
#include "allegroinitialiser.h"
#include "method.h"
#include "image.h"
#include "gui.h"

#include "./rtfb/dda_maker.h"
#include "./crebain/poly_maker.h"
#include "./peakDetect/all_maker.h"
#include "./yvl/mmaker_linear.h"
#include "./yvl/mmaker2d_furje.h"
#include "./sin/mmaker_sin.h"
#include "./cub/mmaker_cub.h"
#include "./qz/neural_maker.h"
#include "./xcub/mmaker_xcub.h"


struct CApplication::SImpl
{
	SImpl () :m_AllegInit (), m_Gui ()
	{
		cout << "CApplication::SImpl::SImpl()" << endl;
		m_Methods.reserve (15);
	}

	~SImpl ()
	{
		vector<IMethod*>::iterator	it;

		for (it=m_Methods.begin (); it!=m_Methods.end (); ++it)
		{
			if (0 != (*it))
			{
				delete (*it);
			}
		}

		cout << "CApplication::SImpl::~SImpl" << endl;
	};

	CAllegroInitialiser	m_AllegInit;
	CGui			m_Gui;
	vector<IMethod*>	m_Methods;
};


CApplication::CApplication ()
{
	m_Impl = new SImpl ();
	cout << "CApplication ()" << endl;
}


CApplication::~CApplication ()
{
	delete m_Impl;
	cout << "~CApplication()" << endl;
}


CApplication* CApplication::createInstance ()
{
	cout << "CApplication::createInstance ()" << endl;
	assert (!"Must be initialized first");
//	init ();
	return 0;
}

void CApplication::deleteInstance (CApplication &instance)
{
	delete &instance;
	cout << "CApplication::deleteInstance ()" << endl;
}

void CApplication::init ()
{
	cout << "CApplicaiton::init ()\n";
	CApplication *app = new CApplication ();
	assert (app);
	assignInstance (*app);
}


int CApplication::run ()
{
	cout << "CApplication::run()" << endl;
	collectMethods ();
	m_Impl->m_Gui.initMenus ();
	m_Impl->m_Gui.doDialog ();

	return 0;
}



void CApplication::processParams (int argc, char *argv[])
{
    char	buff[256] = "\0";

    for (int i=strlen (argv[0])-1; i>0; i--)
    {
	    if ('/' == argv[0][i] || '\\' == argv[0][i])
	    {
		    strncpy (buff, argv[0], i+1);
		    cout << "Path from cwd to executable: " << buff << endl;
		    chdir (buff);
		    break;
	    }
    }
	argc = argc;	// shut up
//	argv = argv;	// shut up
}


void CApplication::registerMethod (IMethod &method)
{
	if (15 <= m_Impl->m_Methods.size ())
	{
		cerr << "Error!" << endl << "Can't register more that 15 methods" << endl << "Sorry :)" << endl;
		return;
	}

	cout << "CApplication::registerMethod:" << endl;
	cout << "registering method \"" << method.name () << "\"." << endl;
	cout << "works with RGB: " << method.isRgb () << "." << endl;
	cout << endl << endl;
	m_Impl->m_Methods.push_back (&method);
}


void CApplication::getCwd (char *buf) const
{
#ifdef DYRKI_WIN32
	_getcwd (buf, 1022);
#elif defined (DYRKI_LINUX)
	getcwd (buf, 1022);
	strcat (buf, "/");
//	cout << "getcwd called! It returned \"" << buf << "\"." << endl;
#endif
}


bool CApplication::runMethod (int num)
{
	bool	retval = false;

	if (num > m_Impl->m_Methods.size () || num < 0)
		return retval;

	cout << "CApplication::runMethod (" << num << ")" << endl;

	try
	{
		CImage source = m_Impl->m_Gui.getSourceImage ();
		double scale = m_Impl->m_Gui.getScaleFactor ();
		CImage destination ((int)(source.width ()*scale), (int)(source.height ()*scale));
		retval = m_Impl->m_Methods[num]->process (destination, source);
		m_Impl->m_Gui.setDestination (destination, m_Impl->m_Methods[num]->isRgb ());
	}
	catch (...)
	{
		cout << "Exception caught!" << endl;
		return false;
	}

	return retval;
}

void CApplication::collectMethods ()
{
	registerMethod (*CDdaMaker::makeMethod ());
	registerMethod (*CPolyMaker::makeMethod ());
	registerMethod (*CAdvPolyMaker::makeMethod ());
	registerMethod (*CLagrangeMaker::makeMethod ());/**/
	registerMethod (*CSEMaker::makeMethod ());
	registerMethod (*CDiffMaker::makeMethod ());
	registerMethod (*CPeakMaker::makeMethod ());/**/
	registerMethod (*CLinearMaker::makeMethod ());/**/
	registerMethod (*CFurje2dMaker::makeMethod ());/**/
	registerMethod (*CSinMaker::makeMethod ());
	registerMethod (*CCubMaker::makeMethod());
	registerMethod (*CNeuralMaker::makeMethod ());/**/
	registerMethod (*CXcubMaker::makeMethod ());
}


int CApplication::numMethods () const
{
	return m_Impl->m_Methods.size ();
}


char* CApplication::methodName (int index)
{
	if (index < 0 || index >= numMethods ())
		return 0;

	cout << "Retrieving name of the method[" << index << "] = " << m_Impl->m_Methods[index]->name () << "." << endl;

	return m_Impl->m_Methods[index]->name ();
}
