
#ifndef __APPLICATION_H_RTFB__
#define __APPLICATION_H_RTFB__

#include "singleton.h"


class IMethod;

/**
 * \class CApplication application.h application.h
 * \author Vytautas Shaltenis, a.k.a. rtfb
 * \date 2002 12 12
 * \version 1.1
 *
 * The heart of all application. Instatintiated as a singleton, this object
 * controls the behaviour of the program almost from the beginning of its
 * execution. Clients (i.e. implementors of the image processing methods)
 * should not care much about this class, as far as they do not have to
 * facilitate any services privoded by this class.
 */
class CApplication :public CSingleton <CApplication>
{
public:
	~CApplication ();
	static void init ();

	/**
	 * An entry point of the application.
	 */
	int run ();

	/**
	 * A way to get use of command line arguments passed to the
	 * application. This should only concern implemetor of \ref CGui.
	 */
	void processParams (int argc, char *argv[]);

	/**
	 * A way to get the path to the working directory of the program.
	 */
	void getCwd (char *buf) const;

	/**
	 * The only way to know for sure the name of the executable.
	 *
	 * \todo: Write the bastard
	 */
	void getExecName (char *buf) const;

	/**
	 * This is how application calls the method execute.
	 *
	 * \note This method is intended to be used only by applcation core
	 * maintainer, so clients may forget it.
	 */
	bool runMethod (int num);

private:
	CApplication ();

	/**
	 * Methods may register through this interface.
	 *
	 * \deprecated It is unlikely that this way of registering is going
	 * to be used, so do not pay too much attention to it.
	 */
	void registerMethod (IMethod &method);

	static CApplication* createInstance ();
	static void deleteInstance (CApplication &instance);
	void collectMethods ();

	int	numMethods () const;
	char* methodName (int index);

//	friend int main (int argc, char *argv[]);	// how comes this doesn't work ???!?
	friend class CSingleton <CApplication>;
	friend class registerer;
	friend class CGui;

	struct	SImpl;

	SImpl	*m_Impl;
};


#endif		// #ifndef __APPLICATION_H_RTFB__
