#ifndef __POLYNOMIAL_H_CREBAIN__
#define __POLYNOMIAL_H_CREBAIN__

#include "../nommgr.h"
#include <string>
#include <iostream>
#include <vector>
#include "../mmgr.h"
#include "../method.h"
#include "../image.h"

using namespace std;

class CRow
 {
  protected:
   vector<long double> coefs;
   int length;

  public:
   CRow () {length = 0;};
   CRow (int len);
//   CRow (const CRow &row);
   int getlength () {return length;};
   virtual ~CRow ();
   int getlen ();
   virtual void print ();
   long double &member (int index);

 };

class CEquation: public CRow
 {
  protected:
   long double result;

  public:
   CEquation () {};
   CEquation (int len, long double res = 0);
//   CEquation (const CEquation &equ);
   virtual ~CEquation ();
   virtual void print ();
   void setresult (const long double res);
   long double getresult ();
   void setcoefs (long double x);
   void add (CEquation &equ);
   void sub (CEquation &equ);
   void mul (const long double scalar);
   void div (const long double scalar);
 };

class CPolynomial: public CEquation
    {
    protected:
	int numofpoints;
    public:
    	CPolynomial () {};
	CPolynomial (int len): CEquation (len, 0) {/*cout << "CPolynomial (int len): CEquation (len, 0)\n";*/};
//	CPolynomial (const CPolynomial &poly): CEquation (poly) {};
	~CPolynomial () {};
	void calculate (long double value);
	void calcdiff (long double value);
	double get_value (double x);
	void print ();

	void pmul (CPolynomial& poly);
    };

class CMatrix
 {
   private:
    vector<CPolynomial> rows;
    int dimension;

   public:
    CMatrix ();
    CMatrix (int dim);
    virtual ~CMatrix ();
    void swaprows (int i, int j);
    void gausselim ();
    vector<long double> getresults ();
    virtual void print ();

    CPolynomial& member (int index);
 };

class CPolyInt: public CPolynomial
	{
	private:
		void construct (SLayer &l, int scanln, bool isVert, int start, int len);
	public:
		CPolyInt () {};
		CPolyInt (SLayer &l, int scanln, bool isVert, int start = 0, int len = -1);
		~CPolyInt () {};
	};

class CAdvPolyInt: public CPolynomial
	{
	private:
		void construct (SLayer &l, int scanln, bool isVert, int start, int len);
	public:
		CAdvPolyInt () {};
		CAdvPolyInt (SLayer &l, int scanln, bool isVert, int start = 0, int len = -1);
		~CAdvPolyInt () {};
	};

class CLagrangeInt: public CPolynomial
	{
	private:
		void construct (SLayer &l, int scanln, bool isVert, int start, int len);
	public:
		CLagrangeInt () {};
		CLagrangeInt (SLayer &l, int scanln, bool isVert, int start = 0, int len = -1);
		~CLagrangeInt () {};
	};

long double plowe (long double *buf, int len, const long double *bla);
long double plowl (long double x, int power, long double bla = 1.0);

class CPolyMethod :public IMethod
{
public:
	CPolyMethod () :IMethod ()
	{
		std::cout << "CPolyMethod" << std::endl;
	}

	~CPolyMethod ()
	{
		std::cout << "~CPolyMethod" << std::endl;
	}

	/**
	 * This function implements an interface implied in IMethod and it
	 * returns the name of the method.
	 */
	char* name () const
	{
		static char the_name[] = "Polynomial";
		return the_name;
	}

	const bool isRgb () const { return true; }

	bool processlayer (SLayer &dest, SLayer &src);
	/**
	 * Processes the image passed and fills the destination image.
	 * \param dest		Destination image
	 * \param src		Source image
	 *
	 * \todo Write the bastard
	 */
	bool process (CImage &dest, CImage &src);

private:
};

class CAdvPolyMethod :public IMethod
{
public:
	CAdvPolyMethod () :IMethod ()
	{
		std::cout << "CAdvPolyMethod" << std::endl;
	}

	~CAdvPolyMethod ()
	{
		std::cout << "~CAdvPolyMethod" << std::endl;
	}

	/**
	 * This function implements an interface implied in IMethod and it
	 * returns the name of the method.
	 */
	char* name () const
	{
		static char the_name[] = "Advanced Polynomial";
		return the_name;
	}

	const bool isRgb () const { return true; }

	/**
	 * Processes the image passed and fills the destination image.
	 * \param dest		Destination image
	 * \param src		Source image
	 *
	 * \todo Write the bastard
	 */
	bool processlayer (SLayer &dest, SLayer &src);
	bool process (CImage &dest, CImage &src);

private:
};

class CLagrangeMethod :public IMethod
{
public:
	CLagrangeMethod () :IMethod ()
	{
		std::cout << "CLagrangeMethod" << std::endl;
	}

	~CLagrangeMethod ()
	{
		std::cout << "~CLagrangeMethod" << std::endl;
	}

	/**
	 * This function implements an interface implied in IMethod and it
	 * returns the name of the method.
	 */
	char* name () const
	{
		static char the_name[] = "Lagrange";
		return the_name;
	}

	const bool isRgb () const { return true; }

	bool processlayer (SLayer &dest, SLayer &src);

	/**
	 * Processes the image passed and fills the destination image.
	 * \param dest		Destination image
	 * \param src		Source image
	 *
	 * \todo Write the bastard
	 */
	bool process (CImage &dest, CImage &src);

private:
};

#endif
	// __POLYNOMIAL_H_CREBAIN__


