
#ifndef __CIMAGE_H_RTFB__
#define __CIMAGE_H_RTFB__

/*
 * \class CImage image.h image.h
 * \author Vytautas Shaltenis, a.k.a. rtfb and Justas Sadzevichius, a.k.a. YvL
 * \date 2002 12 12
 * \version 1.1
 *
 * Our main representation of image data type. There should be
 * no Allegro stuff in your code, only this class and what it
 * provides.
 */
class CImage
{
public:
	/*	SInternalLayer
	 * Raw representation of CImage. Use with caution!
	 * Use it only for fast mathematical calculations, not for
	 * image storing/loading/copying/etc. That's what CImage
	 * is for.
	 */
	struct SInternalLayer
	{
	public:
		int	w;		// width
		int	h;		// height
		double	**data;		// image data
	private:
		SInternalLayer& operator= (const SInternalLayer &src);
		SInternalLayer (const SInternalLayer &src);
		SInternalLayer (int nw, int nh);
		~SInternalLayer ();
		SInternalLayer ();
		friend class CImage;
		friend class ccurve_3drgb;
		friend class ccurve_3dfurje;
		friend class CPolyMethod;
		friend class CAdvPolyMethod;
		friend class CLagrangeMethod;
	};

public:
	CImage (int w, int h);
	CImage (const CImage &src);
	CImage& operator= (const CImage &src);
	~CImage ();

	const int width () const { return m_Width; };
	const int height () const { return m_Height; };

	SInternalLayer& getR () { return m_R; };
	SInternalLayer& getG () { return m_G; };
	SInternalLayer& getB () { return m_B; };
	SInternalLayer& getH () { return m_H; };
	SInternalLayer& getS () { return m_S; };
	SInternalLayer& getV () { return m_V; };


private:
	CImage ();		// hide default constructor

	int			m_Width, m_Height;
	SInternalLayer		m_R, m_G, m_B;
	SInternalLayer		m_H, m_S, m_V;
};

	/*	SLayer
	 * This is *the* image type tu use if you want raw access
	 * to the data. I.e. refer to it wherever you want like this:
	 *	SLayer  &my_layer;
	 * There's no need to refer like
	 *	CImage::SInternalLayer    &my_layer;
	 */
	typedef struct CImage::SInternalLayer SLayer;


#endif		// #ifndef __CIMAGE_H_RTFB__
