
#ifndef __CMETHOD_H__RTFB__
#define __CMETHOD_H__RTFB__

#include "nommgr.h"
#include <string>
#include "mmgr.h"


class CImage;		// forward declaration

/**
 * \class IMethod method.h method.h
 * \author Vytautas Shaltenis, a.k.a. rtfb
 * \date 2002 12 12
 * \version 1.1
 *
 * Main Method interface
 *
 * The very basic, but vital interface for any method that is to be implemented
 * in Igrovod.
 */
class IMethod
{
public:
	IMethod ();

	virtual ~IMethod () = 0;

	/**
	 * The main entry point for the method. Application calls this
	 * method with destination and source images. Scale factor must be
	 * obtained by the method, according to dimensions of source and
	 * destination
	 * \param dest		Destination image
	 * \param src		Source image
	 *
	 * \return True if all went ok, false otherwise.
	 */
	virtual bool process (CImage &dest, CImage &src) = 0;

	/**
	 * This function must return name of the method. That is to be displayed
	 * in the Methods menu, so it must be very short (e.g. Lagrange,
	 * Fourier, or similar).
	 */
	virtual char* name () const = 0;

	/**
	 * This function must return whether the method works with RGB
	 * layers (true), or with HSV (false). The only need for this is
	 * that application would know from which layers to form the final
	 * image that will be displayed after the method does its job.
	 */
	virtual const bool isRgb () const = 0;

};

#endif		// #ifndef __CMETHOD_H__RTFB__
