#include <iostream>
#include <math.h>

#include "../mmgr.h"
#include "pTypes.h"
#include "dfrn_lib.h"


diffr::diffr(mData *src) : diffX(NULL),diffY(NULL),diffB(NULL),DErrTag(false),
       diffFB(NULL),width(0),height(0)
 {
    if (!src) diffrErr("constructor (1) got NULL arguments.");
    width = src->width;
    height = src->height;
    DiffX(src->red, src->green, src->blue);
    DiffY(src->red, src->green, src->blue);
    return;
  }

diffr::~diffr() {
    if (diffX) delete diffX;
    if (diffY) delete diffY;
    if (diffB) delete diffB;
    if (diffFB) delete diffFB;
    return;
  }

void diffr::diffrErr(const char *c) {
    std::cerr << "Error in PeakDetect/diffr_lib: " << c << "\n";
    std::cerr.flush();
    DErrTag = true;
    return;
  }

void diffr::diffrErr() {
    return diffrErr("Error in diffr module.");
  }  
  
void diffr::DiffX(dot **r, dot **g, dot **b) {
    if (!(r && g && b)) return diffrErr("DiffX: got NULL arguments.");
    diffX = new mData(width, height);                 
    for (indx j = 0; j < height; j++) {
         for (indx i = 0; i < width-1; i++) {
            diffX->red[j][i] = r[j][i+1] - r[j][i];
            diffX->green[j][i] = g[j][i+1] - g[j][i];
            diffX->blue[j][i] = b[j][i+1] - b[j][i];
         }
         diffX->red[j][width-1] = diffX->red[j][width-2];
         diffX->green[j][width-1] = diffX->green[j][width-2];
         diffX->blue[j][width-1] = diffX->blue[j][width-2];
    }
    return;
  }  

void diffr::DiffY(dot **r, dot **g, dot **b) {  
  if (!(r && g && b)) return diffrErr("DiffY: got NULL arguments.");
    diffY = new mData(height, width);
    for (indx j = 0; j < width; j++) {
         for (indx i = 0; i < height-1; i++) {
            diffY->red[j][i] = r[i+1][j] - r[i][j];
            diffY->green[j][i] = g[i+1][j] - g[i][j];
            diffY->blue[j][i] = b[i+1][j] - b[i][j];
         }
         diffY->red[j][height-1] = diffY->red[j][height-2];
         diffY->green[j][height-1] = diffY->green[j][height-2];
         diffY->blue[j][height-1] = diffY->blue[j][height-2];
    }
    return;
  }

  void diffr::DiffB() {
    if (!(diffX && diffY)) return diffrErr("DiffB: got NULL arguments");
    if (diffB) return;
    diffB = new mData(width, height);
    for (indx j = 0; j < height; j++) {
         for (indx i = 0; i < width; i++) {
            diffB->red[j][i] = maxfabs(diffX->red[j][i], diffY->red[i][j]);
            diffB->green[j][i] = maxfabs(diffX->green[j][i], diffY->green[i][j]);
            diffB->blue[j][i] = maxfabs(diffX->blue[j][i], diffY->blue[i][j]);
         }
    }
    return;
  }

  void diffr::DiffF() {
    diffFB = new mData(width, height);
    for (indx j = 0; j < height; j++) {
         for (indx i = 0; i < width; i++) {
            diffFB->red[j][i] = fabs(diffB->red[j][i]);
            diffFB->green[j][i] = fabs(diffB->green[j][i]);
            diffFB->blue[j][i] = fabs(diffB->blue[j][i]);
         }
    }
    return;
  }

inline dot diffr::maxfabs(dot a, dot b) {
    return fabs(a) > fabs(b) ? a : b;
};
