
#ifndef DFRN_LIB_H
#define DFRN_LIB_H
#define DFRN_VER "1.0.2-STABLE"

#include "pmData.h"
#include "pTypes.h"

/** \class diffr dfrn_lib.h dfrn_lib.h
    \author Robertas Dumpis
    \date 2002 11 16
    \version 1.0
    \brief A class for image differentiation 
    This class provides image differentiation services for other methods,
    that need it, and it can output image to Dyrki core only if scale factor is set to 1.0 
    Image differentation could be also called edge detection,
    because it emphasizes edges in the image. 
    \b note: \n
    In order to get picture from this method, you need to set scale factor to 1.0 
*/

class diffr {
  
  private:
     indx width, height;
     mData *diffX, *diffY, *diffB, *diffFB;
     bool DErrTag;
     void DiffX(dot **r, dot **g, dot **b);             
     void DiffY(dot **r, dot **g, dot **b);             
     void DiffB();
     void DiffF();
     void diffrErr(const char *c);
     void diffrErr();
     inline dot maxfabs(dot a, dot b);              

  public:
    diffr(mData *src);
    ~diffr();
    mData *getDiffX() { return diffX; };
    mData *getDiffY() { return diffY; };
    mData *getDiff()  {
        if (diffB) return diffB;
        DiffB();
        return diffB;
    };
    mData *getFDiff() {
        if (diffFB) return diffFB;
        DiffB();
        DiffF();
        return diffFB;
    };
    indx getWidth()   { return width; };
    indx getHeight()  { return height; };
    bool getErr()     { return DErrTag; };

};

#endif
