#include <math.h>
#include <iostream>

#include "../mmgr.h"
#include "peak_lib.h"
                                     //       96           48           4
                                     
//Reikia parashyti daygiau konstruktoriu (del perf.)                                     
SimplePeak::SimplePeak(mData *src, int t, indx w, indx h, indx hm) :
    diff(NULL),peaks(NULL),peaksX(NULL),peaksY(NULL),img(NULL),SPErrTag(false) 
  {
    if (!src) {
        SPErr("Constructor: NULL argument received.");
        return;
    }
    width = src->width;
    height = src->height;
    img = new mData(src);
    if (!img) SPErr("Constructor 2: Construction of img failed.");
    rng_w = w;
    rng_h = h/255.0;
    rng_hm = hm/255.0;
    InitSP(t, w, h, hm);
    return;
}

SimplePeak::SimplePeak(CImage &src, int t, indx w, indx h, indx hm) :
    diff(NULL),peaks(NULL),peaksX(NULL),peaksY(NULL),img(NULL),SPErrTag(false)
  {
    width = src.width();
    height = src.height();
    img = new mData(src);
    if (!img) SPErr("Constructor 2: Construction of img failed.");
    rng_w = w;
    rng_h = h/255.0;
    rng_hm = hm/255.0;
    InitSP(t, w, h, hm);
    return;
}

#ifdef PEAK_ALONE
SimplePeak::SimplePeak(GBitmap *src, int t, indx w, indx h, indx hm) :
    diff(NULL),peaks(NULL),peaksX(NULL),peaksY(NULL),img(NULL),SPErrTag(false)
  {
    if (!src) {
        SPErr("Constructor 3: NULL argument received.");
        return;
    }
    width = src->getWidth();
    height = src->getHeight();
    img = new mData(src);
    if (!img) SPErr("Constructor 2: Construction of img failed.");
    rng_w = w;
    rng_h = h/255.0;
    rng_hm = hm/255.0;
    InitSP(t, w, h, hm);
    return;
}
#endif

void SimplePeak::InitSP(int t, indx w, indx h, indx hm) {
    diff = new diffr(img);
    if (!diff) SPErr("SPInit: Construction of diff failed.");
    if (t == 1) Zero = &SimplePeak::zero_detect2;
            else Zero = &SimplePeak::zero_detect;
    FindPeaks();                    //Surandame peak'us (visus)
    if (hm) CorrectWrap(&SimplePeak::CorrectHighPass);      //HighPass filtras
    if (w && h) CorrectWrap(&SimplePeak::CorrectPeaksLine); //CPL filtras
    CombinePeaks();         //Sukombinuojame peak'us y viena image'a
    return;
}

SimplePeak::~SimplePeak() {
    if (peaks) delete peaks;
    if (peaksY) delete peaksY;
    if (peaksX) delete peaksX;
    if (diff) delete diff;
    if (img) delete img;
    return;
}

void SimplePeak::SPErr() {
   return SPErr("Unknown error.");
  }

void SimplePeak::SPErr(const char *c) {
    std::cerr << "Error in PeakDetect/SimplePeak class: " << c << "\n";
    std::cerr.flush();
    SPErrTag = true;
    return;
  }

void SimplePeak::FindPeaks() {
    if (!diff) return SPErr("FindPeaks: got NULL argument.");
    peaksX = new mData(width, height);                      //pagal X
    FindPeaks2(diff->getDiffX(), peaksX, width, height);     
    peaksY = new mData(height, width);                      //pagal Y
    FindPeaks2(diff->getDiffY(), peaksY, height, width);
    return;
}

void SimplePeak::FindPeaks2(mData *src, mData *dst, indx w, indx h) {
    for (indx j = 0; j < h; j++) {
        for (indx i = 1; i < w-1; i++) {
            dst->red[j][i] = (this->*Zero)(src->red[j], i);
            dst->green[j][i] = (this->*Zero)(src->green[j], i);
            dst->blue[j][i] = (this->*Zero)(src->blue[j], i);
        }
        dst->red[j][w-1] = 1.0;   dst->red[j][0] = 1.0;
        dst->green[j][w-1] = 1.0; dst->green[j][0] = 1.0;
        dst->blue[j][w-1] = 1.0;  dst->blue[j][0] = 1.0;
    }
    return;
  }

//detektina ir plokschius peak'us
inline dot SimplePeak::zero_detect(dot *src, indx x) {
    if (src[x] > 0) {
        if (src[x-1] < 0) return 1.0;
        if (src[x-1] > 0) return 0.0;
        if ((x > 1) && (src[x-2]) < 0) return 1.0;
        return 0.0;
    }
    if (src[x] < 0) {
        if (src[x-1] < 0) return 0.0;
        if (src[x-1] > 0) return 1.0;
        if ((x > 1) && (src[x-2] > 0)) return 1.0;
        return 0.0;
    }
    if (src[x-1] < 0) {
        if (src[x+1] > 0) return 1.0;
        if (src[x+1] < 0) return 0.0;
        return 1.0;
    }
    if (src[x-1] > 0) {
        if (src[x+1] < 0) return 1.0;
        if (src[x+1] > 0) return 0.0;
        return 1.0;
    }
    if (src[x-1] == 0) {   
        if (src[x+1] != 0) return 1.0;
        return 0.0;
    }
    SPErr("zero_detect: returning abnormal peak value.");
    return 0.5;
}

//detektina _tik_ sharp peak'us
inline dot SimplePeak::zero_detect2(dot *src, indx x) {
    if (src[x] > 0.0) {
        if (src[x-1] < 0.0) return 1.0;
        return 0.0;
    }
    if (src[x] < 0.0) {
        if (src[x-1] > 0.0) return 1.0;
        return 0.0;
    }
    return 0.0;
}

void SimplePeak::CombinePeaks() {           
    if (!(peaksX && peaksY)) {
        SPErr("CombinePeaks(): NULL arguments");
        return;
    }
    if (peaks) { delete peaks; peaks = NULL; }
    peaks = new mData(width, height);
    for (indx y = 0; y < height; y++) {
        for (indx x = 0; x < width; x++) {
             peaks->red[y][x] = (peaksX->red[y][x] > peaksY->red[x][y]) ? peaksX->red[y][x] : peaksY->red[x][y];
             peaks->green[y][x] = (peaksX->green[y][x] > peaksY->green[x][y]) ? peaksX->green[y][x] : peaksY->green[x][y];
             peaks->blue[y][x] = (peaksX->blue[y][x] > peaksY->blue[x][y]) ? peaksX->blue[y][x] : peaksY->blue[x][y];
        }

    }
    return;
}

void SimplePeak::CorrectWrap(void (SimplePeak::*Correct)(dot *, dot *, indx)) {
    for (indx j = 0; j < height; j++) {        //X atzhvilgiu
        (this->*Correct)(peaksX->red[j], img->red[j], width);
        (this->*Correct)(peaksX->green[j], img->green[j], width);
        (this->*Correct)(peaksX->blue[j], img->blue[j], width);
    }
                                                //Y atzhvilgiu
    dot **tmpR = SwapXY(img->red);              //apverchiame originalius img layerius
    dot **tmpG = SwapXY(img->green);
    dot **tmpB = SwapXY(img->blue);
    for (indx j = 0; j < width; j++) {        
        (this->*Correct)(peaksY->red[j], tmpR[j], height);
        (this->*Correct)(peaksY->green[j], tmpG[j], height);
        (this->*Correct)(peaksY->blue[j], tmpB[j], height);
    }
    free_btm(tmpR, width);
    free_btm(tmpG, width);
    free_btm(tmpB, width);
    return;
  }

void SimplePeak::CorrectHighPass(dot *btm, dot *src, indx range) {
    indx x1 = 0, n = C_seekNextPeak(btm, 1);
    while (n < range-2) {
        if (btm[n] > 0.0) {

             if (fabs(src[x1]-src[n]) < rng_hm)        //jei peak'ai per mazhi
                            btm[n] = 0.0;
        }
        x1 = n;
        n = C_seekNextPeak(btm, x1+1);
    }
    return;
}

//dirba, bet nedirba :(
void SimplePeak::CorrectPeaksLine(dot *btm, dot *src, indx range) {
   // return;
   // SPErr("CPL filter processing");
    indx x1 = 0, x2 = 0;
    indx i = C_seekNextPeak(btm, 0);  //randame pirma peak'a
    while (i < range-1) {
        x1 = i;                        //ishsaugom pries tai buvusi peak'a
        i = C_seekNextPeak(btm, i+1);
        if (i >= range-1) break;
        if (i-x1 < rng_w) {           //tikriausiai bus daugiau peak'u, reikia is ju ishrinkti 'gerus'
             x2 = i;
             i = C_seekNextPeak(btm, i+1);
             if (i >= range-1) break;
             while (i-x1 <= rng_w) {
                 if (fabs(src[i]-src[x1]) > rng_h) {
                     x2 = i;
                     break;
                 }        
                 if (fabs(src[x2]-src[x1]) < fabs(src[i]-src[x1])) x2 = i; //jei naujesnis Peak'as 'geresnis'...
                 i = C_seekNextPeak(btm, i+1);  
                 if (i >= range-1) break;  
             }
             i = x2;
             C_cleanLine(btm, x1+1, i);
        }
        else {   //Peak'as yra vienas 
             C_cleanLine(btm, x1+1, i);
        }    
    }
    return;
}


inline indx SimplePeak::C_seekNextPeak(dot *btm, indx begin) {
    for (indx n = begin; n < width; n++)
            if (btm[n] > 0.0) return n;          
    return width-1;
}

inline void SimplePeak::C_cleanLine(dot *btm, indx from, indx to) {
    for (indx n = from; n < to; n++) 
            btm[n] = 0.0;
}

//suxx
dot **SimplePeak::SwapXY(dot **src) {
    dot **tmp = alloc_btm(height, width);
    for (indx j = 0; j < height; j++) 
        for (indx i = 0; i < width; i++) 
                tmp[i][j] = src[j][i];
    return tmp;
}
