#ifndef PEAK_LIB_H
#define PEAK_LIB_H
#define PEAK_VER "1.0.6-STABLE"

#include "dfrn_lib.h"
#ifdef PEAK_ALONE
#include "gen_bitmap.h"
#endif
#include "pmData.h"
#include "pTypes.h"
#include <stdlib.h>

/** \class SimplePeak peak_lib.h peak_lib.h
    \author Robertas Dumpis
    \date 2002 11 19
    \version 1.0
    \brief A method, which finds peaks in the image
    This method uses straightforward and simple peak finding algorithm.
    Firstly, it finds all the peaks in the image, then filters out some
    local peaks. It was originally developed for helping to train neural
    networks, and the algorithm doesn't always find the best peaks.
    \b note: \n
    In order to get picture from this method, you need to set scale factor to 1.0 
*/

class SimplePeak {

     private:
        diffr *diff;
        mData *img;
        mData *peaks, *peaksX, *peaksY;
        indx width, height;
        dot rng_h, rng_hm;        //Peak korekcijos parametrai
        indx rng_w;
		bool SPErrTag;
        void (SimplePeak::*Correct)(dot *, dot *, indx);
        dot (SimplePeak::*Zero)(dot *, indx);
        void InitSP(int t, indx w, indx h, indx hm);
        void FindPeaks();
        void FindPeaks2(mData *src, mData *dst, indx w, indx h);
        inline dot zero_detect(dot *src, indx x);
        inline dot zero_detect2(dot *src, indx x);
        void CorrectWrap(void (SimplePeak::*Correct)(dot *, dot *, indx));
        dot **SwapXY(dot **src);       //Negerai, bet butina
        void CorrectHighPass(dot *btm, dot *src, indx range);
        void CorrectPeaksLine(dot *btm, dot *src, indx range);
        inline indx C_seekNextPeak(dot *btm, indx begin);
        inline void C_cleanLine(dot *btm, indx from, indx to);
        void CombinePeaks();          
        void SPErr();
        void SPErr(const char *c);
  
    public:
#ifdef PEAK_ALONE
        SimplePeak(GBitmap *src, int t, indx w, indx h, indx hm);  //(absolete)
#endif
        SimplePeak(CImage &src, int t, indx w, indx h, indx hm);
        SimplePeak(mData *src, int t, indx w, indx h, indx hm);
        ~SimplePeak();
        mData *getPeak() { return peaks; };
		void getPeak(CImage &dest) {
            peaks->exportImg(dest);
            if (peaks->ErrTag) SPErrTag = true;
        };
        indx getWidth() { return width; };
        indx getHeight() { return height; };
        bool getErr() { return SPErrTag; };

};

#endif
