#include <iostream>

#include "../mmgr.h"
#include "../image.h"
#include "pmData.h"

//kaip malloc, tik allocate'ina fiksuotam tipui dot**
dot **alloc_btm(indx w, indx h) {
    dot **tmp = NULL;
    indx i;
    if (!(w && h)) return NULL;
    tmp = new dot*[h];
    for (i = 0; i < h; i++) {
            if (!(tmp[i] = new dot[w])) {
                std::cerr << "alloc_btm: allocation of memory failed. Retrying...\n" << std::cerr.flush();
                for (int j = 0; j < 100; j++) {
                    if ((tmp[i] = new dot[w])) break;
                }
                if (!tmp[i]) goto FAILURE;
            }
    }
    return tmp;
    FAILURE:  //Tikru tikriausias nesusipratimas
    std::cerr << "alloc_btm: cannot allocate memory. Giving up.\n" << std::cerr.flush();
    free_btm(tmp, i);
    return NULL;
}

void free_btm(dot **tmp, indx h) {
    if (!tmp) {
        std::cerr << "free_btm: got null argument. Giving up.\n" << std::cerr.flush();
        return;
    }
    for (indx j = 0; j < h; j++) delete []tmp[j];
    delete []tmp;
    return;
}

dot **cpy_btm(dot **src, indx w, indx h) {
    if (!src) {
        std::cerr << "cpy_btm: got null argument. Giving up.\n" << std::cerr.flush();
        return NULL;
    }
    dot **dst = alloc_btm(w, h);
    if (!dst) {
        std::cerr << "cpy_btm: cannot allocate memory. Giving up.\n" << std::cerr.flush();
        return NULL;
    }
    for (indx j = 0; j < h; j++) {
        for (indx i = 0; i < w; i++) {
            if (fabs(src[j][i]) > 1.0) {
                std::cerr << "cpy_btm: invalid value in source.\n"; std::cerr.flush();
        //        return NULL;    //bomba
            }
            dst[j][i] = src[j][i];
        }
    }
    return dst;
  }

dot **cpy_btmC(dot **src, indx w, indx h) {
    if (!src) {
        std::cerr << "cpy_btmC: got null argument. Giving up.\n" << std::cerr.flush();
        return NULL;
    }
    dot **dst = alloc_btm(w, h);
    if (!dst) {
        std::cerr << "cpy_btmC: cannot allocate memory. Giving up.\n" << std::cerr.flush();
        return NULL;
    }
    for (indx j = 0; j < h; j++) {
        for (indx i = 0; i < w; i++) {
            if (fabs(src[i][j]) > 1.0) {
                std::cerr << "cpy_btm: invalid value in source.\n";
         //       return NULL;    //bomba
            }
            dst[j][i] = src[i][j];
        }
    }
    return dst;
  }  
  

mData::mData(indx w, indx h) :
    red(NULL),green(NULL),blue(NULL),hue(NULL),sat(NULL),value(NULL),ErrTag(false)
{
    HSV=false;
    width = w; height = h;
    red = alloc_btm(width, height);
    green = alloc_btm(width, height);
    blue = alloc_btm(width, height);
    if (!(red && green && blue)) MDErr("Constructor: unable to allocate memory.");
    return;
  }

mData::mData(CImage &some) :
    red(NULL),green(NULL),blue(NULL),hue(NULL),sat(NULL),value(NULL),ErrTag(false)
{
    HSV=false;
    width = some.width(); height = some.height();
    red = cpy_btmC(some.getR().data, width, height);       
    green = cpy_btmC(some.getG().data, width, height);     
    blue = cpy_btmC(some.getB().data, width, height);
    if (!(red && green && blue)) MDErr("Constructor 2: unable to allocate memory.");    
    return;
  }
 
#ifdef PEAK_ALONE
mData::mData(GBitmap *some) :
    red(NULL),green(NULL),blue(NULL),hue(NULL),sat(NULL),value(NULL),ErrTag(false)
 {
    HSV=false;
    width = some->getWidth(); height = some->getHeight();
    red = cpy_btm(some->getR(), width, height);
    green = cpy_btm(some->getG(), width, height);
    blue = cpy_btm(some->getB(), width, height);
    if (!(red && green && blue)) MDErr("Constructor 3: unable to allocate memory.");  
    return;
  }
#endif
  
 mData::mData(mData *some) :
    red(NULL),green(NULL),blue(NULL),hue(NULL),sat(NULL),value(NULL),ErrTag(false)
{
    if (!some) {
        MDErr("constructor 4: got NULL argument.");
        return;
    }
    HSV = some->HSV;
    width = some->width; height = some->height;
    if (some->red) red = cpy_btm(some->red, width, height);
    if (some->green) green = cpy_btm(some->green, width, height);
    if (some->blue) blue = cpy_btm(some->blue, width, height);
    if (!(red && green && blue)) MDErr("Constructor 4: unable to allocate memory.");  
    return;
}
   
mData::~mData() {
   if (red) free_btm(red, height);
   if (green) free_btm(green, height);
   if (blue) free_btm(blue, height);
   if (hue) free_btm(hue, height);
   if (sat) free_btm(sat, height);
   if (value) free_btm(value, height);
   return;
  }

void mData::MDErr(const char *c) {
    std::cerr << "Error in PeakDetect/mData: " << c << "\n";
    std::cerr.flush();
    ErrTag = true;
    return;
}

void mData::MDErr() {
    MDErr("Unknown Error");
    return;
}
  
#ifdef PEAK_ALONE
GBitmap *mData::exportImg() {    
    if (!(red && green && blue)) {
        MDErr("exportImg(): no source data");
        return NULL;
    }
    dot **tmpR = cpy_btm(red, width, height);
    dot **tmpG = cpy_btm(green, width, height);
    dot **tmpB = cpy_btm(blue, width, height);
    return new GBitmap(tmpR, tmpG, tmpB, width, height);
  }
#endif
  
void mData::exportImg(CImage &some) {
    if (!(red && green && blue)) {
        MDErr("exportImg() 2: no source data");
        return;
    }
    if ((width != some.width()) || (height != some.height())) {
        MDErr("exportImg() 2: source and destination dimensions not equal.");
        return;
    }
    SLayer &CImgR = some.getR();  //Susikuriame referencus y CImage'a
    SLayer &CImgG = some.getG();
    SLayer &CImgB = some.getB();
    for (indx j = 0; j < height; j++) {
        for (indx i = 0; i < width; i++) {
            CImgR.data[i][j] = red[j][i];
            CImgG.data[i][j] = green[j][i];
            CImgB.data[i][j] = blue[j][i];
        }
    }
    return;
  }

  
