#include <math.h>
#include <iostream>

#include "../mmgr.h"
#include "scale_edge.h"


inline int r_round(double x) {
    return (int)(x+0.5);
}

inline int r_trunc(double x) {
    return (int)(x);
}


ScaleEdge::ScaleEdge(CImage &some, CImage &dome, double kT) :
    src(NULL),dst(NULL),usingMD(false),diff(NULL),SEErrTag(false),
    dCX(0),dCY(0),dDX(0),dDY(0),dRX(0),dRY(0),lX(0.0),lY(0.0),
    widthS(0),heightS(0),widthD(0),heightD(0),factorX(0.0),factorY(0.0)
{
    src = new mData(some);
    widthS = some.width(); heightS = some.height();
    widthD = dome.width(); heightD = dome.height();
    factorX = (double)(widthD)/(double)widthS;
    factorY = (double)(heightD)/(double)heightS;
    koefT = kT;
    diff = new diffr(src);
    if (!diff) {
        SEErr("Constructor: Failed to create diff object. Giving up");
        return;
    }
    dst = new mData(widthD, heightD);
    if (!dst) {
        SEErr("Constructor: Failed to allocate memory. Giving up");
        return;
    }
    doThings();
    return;
}

ScaleEdge::ScaleEdge(mData *some, indx w, indx h, double kT) :
    src(NULL),dst(NULL),usingMD(true),diff(NULL),SEErrTag(false),
    dCX(0),dCY(0),dDX(0),dDY(0),dRX(0),dRY(0),lX(0.0),lY(0.0),
    widthS(0),heightS(0),widthD(w),heightD(h),factorX(0.0),factorY(0.0)
{
//    SEErr("constr. started.");
    if (!some) {
        SEErr("Constructor 2: got NULL argument.");
        return;
    }
    src = some;
    widthS=some->width; heightS=some->height;
    factorX = (double)(widthD)/(double)widthS;
    factorY = (double)(heightD)/(double)heightS;
    koefT = kT;
    diff = new diffr(src);
    if (!diff) {
        SEErr("Constructor 2: Failed to create diff object. Giving up");
        return;
    }
    dst = new mData(widthD, heightD);
    if (!dst) {
        SEErr("Constructor 2: Failed to allocate memory. Giving up");
        return;
    }
    doThings();
    return;
}

ScaleEdge::~ScaleEdge() {
    if (src && !usingMD) delete src;
    if (dst) delete dst;
    if (diff) delete diff;
    return;
}
  
void ScaleEdge::SEErr(const char *c) {
    std::cerr << "Error in ScaleEdge: " << c << std::endl;
//    std::cerr << "C " << dCX << " " << dCY << " D " << dDX << " " << dDY << " R ";
//    std::cerr << dRX << " " << dRY << " l " << lX << " " << lY << std::endl;
//    std::cerr << "[]dX: " << src->red[dCX][dCY] << " []diff:" << diff->getDiffX()->red[dRX][dRY] << std::endl;
    std::cerr.flush();
    SEErrTag = true;
    return;
}

void ScaleEdge::SEErr() {
    SEErr("Unknown error");
    return;
}

void ScaleEdge::doThings() {
    for (indx j = 0; j < heightD; j++) {
        for (indx i = 0; i < widthD; i++) {
            dst->red[j][i] = countValue(src->red, diff->getDiffX()->red, diff->getDiffY()->red, i, j);
            dst->green[j][i] = countValue(src->green, diff->getDiffX()->green, diff->getDiffY()->green, i, j);
            dst->blue[j][i] = countValue(src->blue, diff->getDiffX()->blue, diff->getDiffY()->blue, i, j);
        }
    }
    return;
}
  
inline dot ScaleEdge::countValue(dot **layer, dot **diffX, dot **diffY, indx wD, indx hD) {
    dot x, y;
    getE(wD, hD);                 //Randa taska is src, kurio aplinkoje yra (wD,hD)
/*chia buvo uzhsetinta dCX, dCY, dDX, dDY, dRX, dRY, lX, lY */
    x = weight(layer[dCY][dCX], layer[dCY][dDX], diffX[dCY][dRX], lX);
    y = weight(layer[dCY][dCX], layer[dDY][dCX], diffY[dCX][dRY], lY);
    if (x > 1.0 || y > 1.0) SEErr("countValue : xy out of range, internal error.");
    return fin221(x, y);   
}

inline void ScaleEdge::getE(indx wD, indx hD) {
    dCX = r_trunc((((double)wD/factorX)+(((double)(wD+1))/factorX))/2);
    dCY = r_trunc((((double)hD/factorY)+(((double)(hD+1))/factorY))/2);
    if (dCX >= widthS || dCY >= heightS) {
        SEErr("getE : dC out of range, internal error.");
        std::cerr << dCX << " " << dCY << "\n" << wD << " " << hD << "\n" << factorX << "\n";
        std::cerr.flush();
    }
    lX = fabs((((((double)wD/factorX)+(((double)(wD+1))/factorX))/2) - (double)dCX) - 0.5 )*2;
    lY = fabs((((((double)hD/factorY)+(((double)(hD+1))/factorY))/2) - (double)dCY) - 0.5 )*2;
    if (lX > 1.0 || lY > 1.0) SEErr("getE : l out of range, internal error.");
    dDX = findNeighs(dCX, wD, widthS, factorX);
    dDY = findNeighs(dCY, hD, heightS, factorY);
    if (dDX >= widthS || dDY >= heightS) SEErr("getE : dD out of range, internal error.");
    dRX = (dDX > dCX) ? dCX : dDX; 
    dRY = (dDY > dCY) ? dCY : dDY;
    if (dRX >= widthS || dRY >= heightS) SEErr("getE : dR out of range, internal error.");
}

inline indx ScaleEdge::findNeighs(indx base, indx w, indx maxl, double factor) {
    if (r_round((double)w/factor) > base) {
       if (base < (maxl-1)) return (base + 1);
       else return (base); //base -1 ??
    }
    else {
        if (base > 1) return (base - 1);
        else return (base);  //base +1 ??
    }
}

inline dot ScaleEdge::weight(dot s1, dot s2, dot df, double l) {
    double f = (1/(1+exp((l-1)*(sqrt(fabs(df)))*koefT)));
  //  double f = (1/(1+exp((l-0.5)*(fabs(df))*koefT)));
    if (fabs(df) > 1.0) SEErr("weight(): df out of range, internal error.");
    if (fabs(s1) > 1.0) SEErr("weight(): s1 out of range, internal error.");
    if (fabs(s2) > 1.0) SEErr("weight(): s2 out of range, internal error.");
    return (s2*(1-f) + s1*f);
}

/*
inline double ScaleEdge::transf(dot df, double l, bool direct) {
  //  if (direct) return (1/(1+exp((l-0.5)*fabs(df)*koefT)));
    return (1/(1+exp((l-0.5)*fabs(df)*koefT)));
}
 */
//TODO: find better function to implement this
inline dot ScaleEdge::fin221(dot x, dot y) {
  //  return hypot(x,y)/1.42;
    return (x+y)/2;   
}

void ScaleEdge::exportScale(CImage &ddd) {
    dst->exportImg(ddd);
    return;
}

mData *ScaleEdge::exportScale() {
    return new mData(dst);
}

mData *ScaleEdge::getScale() {
    return dst;
}
