#ifndef __SE_MAKER_H_
#define __SE_MAKER_H_

#include "../pf/method_maker.h"
#include "se_wrap.h"

#include <iostream>			// DEBUG



/**
 * This is the joint of method and application. Every method must create a
 * class just like this in order to get connected to application. If one
 * has a method, but does not have a "maker" like this, there's no way to
 * get registered with application, so such method may not be functional.\n
 * All the interfaces provided in \ref CMethodMaker
 * must be implemented exactly like shown in this example, otherwise it
 * will not compile, or -- even worse -- will crash painfully.\n
 * \b Important \b note: \n
 * Don't forget to call a static makeMethod() in \ref CApplication::collectMethods
 */
 
class CSEMaker :public CMethodMaker
{
public:
	/**
	 * Destructor. Sestructors of makers should do nothing.
	 */
	~CSEMaker ()
	{
		std::cout << "~CSEMaker" << std::endl;
	}

	/**
	 * Default constructor. Should do nothing too.
	 */
	CSEMaker () :CMethodMaker ()
	{
		std::cout << "CSEMaker" << std::endl;
	}

	/**
	 * The method instantiator. The main function and the purpose of
	 * the Pluggable Factory pattern. The whole purpose of this function
	 * is to instantiate the method and return a pointer to it. The only
	 * activity this function should perform is the
	 * \code return new CSomeMethod (); \endcode
	 */
	static IMethod* makeMethod ()
	{
		std::cout << "CSEMaker::makeMethod" << std::endl;
		return new CSEMethod ();
	}
};

#else
#error RTFM!
#endif
