#include <iostream>
#include <vector>
#include <math.h>
#include "mmgr.h"
#include "layer.h"

using namespace std;

#define SGN(x) ( ((x<0) ? (-1) : (1)) )
#define MAX(a,b) ( (a<b) ? (b) : (a) )
#define MIN(a,b) ( (a>b) ? (a) : (b) )
#define sqr(a) (a*a)

#define INCREASE_RATE 1.2
#define DECREASE_RATE 0.5
// neveikia MAX_W
#define MAX_W 5000.0
// neveikia MAX_W
#define MAX_DELTA 50.0
#define MIN_DELTA 0.0000001
#define INIT_DELTA 0.01

double c_layer::get_output()
{
	double temp=*outputs[0];
	return temp;
}


c_layer::c_layer(int neuron_count)
{
	this->neuron_count = neuron_count;
	for(int i=0;i<neuron_count+1;i++)
	{
		errors.push_back(new double(0.0));
		sum_of_errors.push_back(new double(0.0));
		if (i==neuron_count+1) outputs.push_back(new double(1.0));
			else outputs.push_back(new double(0.0));
	}
}

c_layer::~c_layer()
{
	for(int i=0;i<neuron_count+1;i++)
	{
		delete errors[i];
		delete sum_of_errors[i];
 		delete outputs[i];
	}
}

void c_layer::connect(c_layer * previous_layer)
{
	previous_layer_neuron_count = previous_layer->neuron_count+1;

	for(int i=0;i<previous_layer_neuron_count;i++)
	{
		previous_layer_errors.push_back(previous_layer->errors[i]);
		sum_of_previous_layer_errors.push_back(previous_layer->sum_of_errors[i]);
		inputs.push_back(previous_layer->outputs[i]);
		for(int j=0;j<neuron_count;j++)
		{
			double tmp = (4.0 - 8.0 * (rand() / RAND_MAX + 1.0)) / (double)previous_layer_neuron_count;
//			double tmp = (1.0 - 2.0*(rand() / (RAND_MAX + 1.0)))/sqrt((double)previous_layer_neuron_count);
//			if (tmp>0) tmp+=1.0; else tmp-=1.0;
			weights.push_back(tmp);
			deltas.push_back(INIT_DELTA);
			derivatives.push_back(0.0);
			previous_iteration_derivatives.push_back(0.0);
		}
	}
}

double c_layer::function(double param)
{
	return 1.0 / (1.0 + exp(-param));
}

double c_layer::fderivative(double param)
{
	return param*(1.1-param);
}

/*/

double c_layer::function(double param)
{
//	param = 2*((((int)param + 10 ) / 20)%2) * (-(param + 20*(int)(((int)param + 10 ) / 20))) + (param + 20*(int)(((int)param + 10 ) / 20));
	return 2.0 / (1.0 + exp(-param)) - 1.0;
}

double c_layer::fderivative(double param)
{
//	param = 2*((((int)param + 10 ) / 20)%2) * (-(param + 20*(int)(((int)param + 10 ) / 20))) + (param + 20*(int)(((int)param + 10 ) / 20));
	return 1.0-sqr(param);
}
 */
void c_layer::forward()
{
	double temp;
	for(int i=0;i<neuron_count;i++)
	{
		temp=0.0;
		for(int j=0;j<previous_layer_neuron_count;j++)
		{
			temp += *(inputs[j]) * weights[i*previous_layer_neuron_count+j];
		}
		*(outputs[i]) = function(temp);
	}
}


double c_layer::calculate_error(vector<double> target)
{
	double global_error=0.0;
	for (int i=0;i<neuron_count;i++)
	{
		*(errors[i]) = (target[i] - *(outputs[i]))
				* fderivative(*(outputs[i]));
		if ((target[i] - *(outputs[i])) > 0) global_error += (target[i] - *(outputs[i]));
			else global_error -= (target[i] - *(outputs[i]));
	}
	return global_error;
}

void c_layer::backward()
{
	for(int i=0;i<previous_layer_neuron_count;i++)
	{
		*(previous_layer_errors[i]) = 0.0;
		for(int j=0;j<neuron_count;j++)
		{
			*(previous_layer_errors[i]) += *(errors[j]) * weights[j*previous_layer_neuron_count + i];
		}
		*(previous_layer_errors[i]) = *(previous_layer_errors[i])
			* fderivative(*(inputs[i]));
	}
}

void c_layer::batch()
{
	for(int i=0;i<neuron_count;i++)
	{
		for(int j=0;j<previous_layer_neuron_count;j++)
		{
			derivatives[i*previous_layer_neuron_count+j]-= *(errors[i]) * fderivative(*(inputs[j]));
		}
	}
}

void c_layer::clear()
{
	for(int i=0;i<neuron_count;i++)
	{
		for(int j=0;j<previous_layer_neuron_count;j++)
		{
			derivatives[i*previous_layer_neuron_count+j]=0.0;
		}
	}
}

void c_layer::update()
{
	double delta=0.0;
	double deltaw=0.0;
	for(int i=0;i<neuron_count;i++)
	{
		for(int j=0;j<previous_layer_neuron_count;j++)
		{
			int crnt_wght=i*previous_layer_neuron_count + j;
			if (previous_iteration_derivatives[crnt_wght]
				*derivatives[crnt_wght] > 0)
			{
				delta = deltas[crnt_wght]*INCREASE_RATE;
				if (delta>MAX_DELTA) delta=MAX_DELTA;
				deltaw = -SGN(derivatives[crnt_wght])*delta;
				weights[crnt_wght] += deltaw;
			}
			else if (previous_iteration_derivatives[crnt_wght]
				*derivatives[crnt_wght] < 0)
			{
				delta = deltas[crnt_wght]*DECREASE_RATE;
				if (delta<MIN_DELTA) delta=MIN_DELTA;
				weights[crnt_wght] -= deltas[crnt_wght];
				derivatives[crnt_wght] = 0.0;
			}
			else
			{
				delta = deltas[crnt_wght];
				deltaw = -SGN(derivatives[crnt_wght])*delta;
				weights[crnt_wght] += deltaw;
			}
//			
			if (weights[crnt_wght]>MAX_W) weights[crnt_wght]=MAX_W;
			if (weights[crnt_wght]<-MAX_W) weights[crnt_wght]=-MAX_W;
//			weights[crnt_wght]=MIN(MAX_W,weights[crnt_wght]);
//			weights[crnt_wght]=MAX(-MAX_W,weights[crnt_wght]);
//			
			deltas[crnt_wght] = delta;
			previous_iteration_derivatives[crnt_wght] = derivatives[crnt_wght];
		}
	}
}

void c_layer::set_outputs(vector<double> data)
{
	int j=0;

	for(std::vector<double*>::iterator i=outputs.begin();
		i!=outputs.end();i++)
	{
		*(*i)=data[j];
		j++;
	}
}
