#include <iostream>
#include <vector>
#include "mmgr.h"
#include "layer.h"
#include "network.h"

#define NEURONS_IN_HIDDEN_LAYER 25

using namespace std;

neural_network::neural_network(int input_count,int hidden_layer_neuron_count,
			int hidden_layer_count,int output_count)
{
	layers.push_back(new c_layer(input_count));

	for(int i=0;i<hidden_layer_count;i++)
	{
		layers.push_back(new c_layer(hidden_layer_neuron_count));
	}

	layers.push_back(new c_layer(output_count));

	for(Iter i=layers.begin()+1;i!=layers.end();i++)
	{
		(*i)->connect(*(i-1));
	}
}

neural_network::neural_network(int hidden_layer_count)
{
	layers.push_back(new c_layer(2));

	for(int i=0;i<hidden_layer_count;i++)
	{
		layers.push_back(new c_layer(NEURONS_IN_HIDDEN_LAYER));
	}

	layers.push_back(new c_layer(1));

	for(Iter i=layers.begin()+1;i!=layers.end();i++)
	{
		(*i)->connect(*(i-1));
	}

}

neural_network::~neural_network()
{
	for(Iter i=layers.begin();i != layers.end();i++)
	{
		if (0 != (*i))
		{
			delete (*i);
		}
	}
}

double neural_network::get_output(double x,double y)
{
	vector<double> input_pattern;
	double temp;

	input_pattern.push_back(x);
	input_pattern.push_back(y);

	layers[0]->set_outputs(input_pattern);

	for(Iter i=layers.begin()+1;i != layers.end();i++)
	{
		(*i)->forward();
	}

	temp = layers.back()->get_output();

	return temp;
}

double neural_network::get_output(double x)
{
	vector<double> input_pattern;
	double temp;

	input_pattern.push_back(x);

	layers[0]->set_outputs(input_pattern);

	for(Iter i=layers.begin()+1;i != layers.end();i++)
	{
		(*i)->forward();
	}

	temp = layers.back()->get_output();

	return temp;
}

double neural_network::batch_pattern(vector<double> inputs,vector<double> outputs)
{
	double temp;

	layers[0]->set_outputs(inputs);

	for(Iter i=layers.begin()+1;i != layers.end();i++)
	{
		(*i)->forward();
	}

	temp = layers.back()->calculate_error(outputs);

	for(RIter i=layers.rbegin();i != layers.rend()-1;i++)
	{
		(*i)->backward();
	}

	for(Iter i=layers.begin();i != layers.end();i++)
	{
		(*i)->batch();
	}
	return temp;
}

double neural_network::batch_pattern(double x,double y,double value)
{
	vector<double> input_pattern;
	vector<double> output_pattern;
	double temp;

	input_pattern.push_back(x);
	input_pattern.push_back(y);
	output_pattern.push_back(value);

	layers[0]->set_outputs(input_pattern);

	for(Iter i=layers.begin()+1;i != layers.end();i++)
	{
		(*i)->forward();
	}

	temp = layers.back()->calculate_error(output_pattern);

	for(RIter i=layers.rbegin();i != layers.rend()-1;i++)
	{
		(*i)->backward();
	}

	for(Iter i=layers.begin();i != layers.end();i++)
	{
		(*i)->batch();
	}
	return temp;
}

void neural_network::train()
{
	for(Iter i=layers.begin()+1;i != layers.end();i++)
	{
		(*i)->update();
	}
}

void neural_network::clear()
{
	for(RIter i=layers.rbegin();i != layers.rend()-1;i++)
	{
		(*i)->clear();
	}
}
