
#ifndef __DDA_MAKER_H_RTFB__
#define __DDA_MAKER_H_RTFB__

#include "../pf/method_maker.h"
#include "dda.h"

#include <iostream>			// DEBUG



/**
 * \class CDdaMaker dda_maker.h rtfb/dda_maker.h
 * \author Vytautas Shaltenis, a.k.a. rtfb
 * \date 2002 11 22
 * \version 1.0
 *
 * This is the joint of method and application. Every method must create a
 * class just like this in order to get connected to application. If one
 * has a method, but does not have a "maker" like this, there's no way to
 * get registered with application, so such method may not be functional.\n
 * All the interfaces provided in \ref CMethodMaker
 * must be implemented exactly like shown in this example, otherwise it
 * will not compile, or -- even worse -- will crash painfully.\n
 * \b Important \b note: \n
 * Don't forget to call a static makeMethod() in \ref CApplication::collectMethods
 */
class CDdaMaker :public CMethodMaker
{
public:
	/**
	 * Destructor. Sestructors of makers should do nothing.
	 */
	~CDdaMaker ()
	{
		std::cout << "~CDdaMaker" << std::endl;
	}

	/**
	 * Default constructor. Should do nothing too.
	 */
	CDdaMaker () :CMethodMaker ()
	{
		std::cout << "CDdaMaker" << std::endl;
	}

	/**
	 * The method instantiator. The main function and the purpose of
	 * the Pluggable Factory pattern. The whole purpose of this function
	 * is to instantiate the method and return a pointer to it. The only
	 * activity this function should perform is the
	 * \code return new CSomeMethod (); \endcode
	 */
	static IMethod* makeMethod ()
	{
		std::cout << "CDdaMaker::makeMethod" << std::endl;
		return new CDdaMethod ();
	}
};

#else
#error Your include guard matches the one of someone else! Change it.
#endif			// #ifndef __DDA_MAKER_H_RTFB__
