
#ifndef __SINGLETON_H_RTFB__
#define __SINGLETON_H_RTFB__

/**
 *	\class CSingleton singleton.h singleton.h
 *	\author Vytautas Shaltenis, a.k.a. rtfb
 *	\version 1.0
 *	\date 2002 11 03
 *	\brief singleton pattern templated implementation
 *	This is a classic implementation of singleton design pattern,
 * implemented as template. The class you want to make a singleton should
 * inherit from it and implement abstract methods. \ref CApplication
 */
template <class T>
class CSingleton
{
public:
	static T& getInstance ()
	{
		if (!m_SingleInstance)
		{
			m_SingleInstance = T::createInstance ();
		}

		return *m_SingleInstance;
	}

	static void finalize ()
	{
		if (!m_SingleInstance)
			return;

		T::deleteInstance (*m_SingleInstance);
		m_SingleInstance = 0;
	}

protected:
	static void assignInstance (T &instance)
	{
		m_SingleInstance = &instance;
	}

private:
	static T	*m_SingleInstance;
};

template <class T>
T *CSingleton<T>::m_SingleInstance = 0;


#endif		// #ifndef __SINGLETON_H_RTFB__
