
#include <math.h>

#include "allegro_hack.h"

#include "mmgr.h"

#include "utils.h"
#include "image.h"

/*
========================================================
//
//        TODO   TODO    TODO   TODO
//
========================================================
*/

static void hsvToRgb(double h, double s, double v, int *r, int *g, int *b);
//static void rgbToHsv(int r, int g, int b, double *h, double *s, double *v);




namespace utils
{

void extractR (SLayer &r, ALLEGRO_BITMAP *bmp)
{
	int		x, y;
	long int	color;
	unsigned char	rp;

	for (y=0; y<bmp->h; y++)
	{
		for (x=0; x<bmp->w; x++)
		{
			color = getpixel (bmp, x, y);
			rp = getr32 (color);
			r.data[x][y] = ((double)rp)/255;
		}
	}
}


void extractG (SLayer &g, ALLEGRO_BITMAP *bmp)
{
	int		x, y;
	long int	color;
	unsigned char	gp;

	for (y=0; y<bmp->h; y++)
	{
		for (x=0; x<bmp->w; x++)
		{
			color = getpixel (bmp, x, y);
			gp = getg32 (color);
			g.data[x][y] = ((double)gp)/255;
		}
	}
}


void extractB (SLayer &b, ALLEGRO_BITMAP *bmp)
{
	int		x, y;
	long int	color;
	unsigned char	bp;

	for (y=0; y<bmp->h; y++)
	{
		for (x=0; x<bmp->w; x++)
		{
			color = getpixel (bmp, x, y);
			bp = getb32 (color);
			b.data[x][y] = ((double)bp)/255;
		}
	}
}


void extractH (SLayer &h, ALLEGRO_BITMAP *bmp)
{
	bmp = (ALLEGRO_BITMAP*) &h;	// shut up the "unused parameter" warning
}


void extractS (SLayer &s, ALLEGRO_BITMAP *bmp)
{
	bmp = (ALLEGRO_BITMAP*) &s;	// shut up the "unused parameter" warning
}


void extractV (SLayer &v, ALLEGRO_BITMAP *bmp)
{
	bmp = (ALLEGRO_BITMAP*) &v;	// shut up the "unused parameter" warning
}


void composeImageFromRGB (ALLEGRO_BITMAP *bmp, CImage &img)
{
	SLayer &tlr = img.getR(), &tlg = img.getG(), &tlb = img.getB();

	for (int y=0; y<img.height(); y++)
	{
		for(int x=0; x<img.width(); x++)
		{
			putpixel(bmp, x, y, makecol32((int)(tlr.data[x][y]*255), (int)(tlg.data[x][y]*255), (int)(tlb.data[x][y]*255)));
		}
	}
}


void composeImageFromHSV (ALLEGRO_BITMAP *bmp, CImage &img)
{
	SLayer	&tlh = img.getH(), &tls = img.getS(), &tlv = img.getV();
	int	r, g, b;

	for (int y=0; y<bmp->h; y++)
	{
		for(int x=0; x<bmp->h; x++)
		{
			hsvToRgb((tlh.data[x][y])*255, (tls.data[x][y])*255, (tlv.data[x][y])*255, &r, &g, &b);
			putpixel(bmp, x, y, makecol(r, g, b));
		}
	}
}


// TODO:
// remove it from here, and put it into CVersatileMaker
unsigned long int uniqueNumber ()
{
	static unsigned long int generator = 0;

	return generator++;
}

};

void hsvToRgb(double h, double s, double v, int *r, int *g, int *b)
{
   double f, x, y, z;
   int i;

   v *= 255.0;

   if (s == 0.0) {
      *r = *g = *b = (int)v;
   }
   else {
      while (h < 0)
	 h += 360;
      h = fmod(h, 360) / 60.0;
      i = (int)h;
      f = h - i;
      x = v * (1.0 - s);
      y = v * (1.0 - (s * f));
      z = v * (1.0 - (s * (1.0 - f)));

      switch (i) {
	 case 0: *r = (int)v; *g = (int)z; *b = (int)x; break;
	 case 1: *r = (int)y; *g = (int)v; *b = (int)x; break;
	 case 2: *r = (int)x; *g = (int)v; *b = (int)z; break;
	 case 3: *r = (int)x; *g = (int)y; *b = (int)v; break;
	 case 4: *r = (int)z; *g = (int)x; *b = (int)v; break;
	 case 5: *r = (int)v; *g = (int)x; *b = (int)y; break;
      }
   }
}


/*
void rgbToHsv(int r, int g, int b, double *h, double *s, double *v)
{
   double min, max, delta, rc, gc, bc;

   rc = (double)r / 255.0;
   gc = (double)g / 255.0;
   bc = (double)b / 255.0;
   max = MAX(rc, MAX(gc, bc));
   min = MIN(rc, MIN(gc, bc));
   delta = max - min;
   *v = max;

   if (max != 0.0)
      *s = delta / max;
   else
      *s = 0.0;

   if (*s == 0.0) {
      *h = 0.0;
   }
   else {
      if (rc == max)
	 *h = (gc - bc) / delta;
      else if (gc == max)
	 *h = 2 + (bc - rc) / delta;
      else if (bc == max)
	 *h = 4 + (rc - gc) / delta;

      *h *= 60.0;
      if (*h < 0)
	 *h += 360.0;
    }
}/**/
