/** \file DyrkiView.cc
    \author Robertas Dumpis
    \date 2002 12 13
    \version 0.9.13
    \brief A part of external viewer utility.
 This file is a part of DyrkiView, utility used for scaled image output
 on a separate window. DyrkiView is a separate executable, whick is optional
 to Dyrki project. It also compiles separately. DyrkiView currently compiles
 and runs only on linux and windows platforms.\n
 This part of the DirkiView is responsible for displaying an image on the
 screen.\n
 \b Note:\n
 This file has nothing to do with dyrki project, except it is used by the project.
 */
 
#include "../allegro_hack.h"
#include <stdio.h>
#ifdef DV_WIN32
#include <strings.h>
#else 
#include <string.h>
#endif

//#include "mmgr.h"

#include "DyrkiView.h"

bool uzhdaryt;
static int img_x, img_y;    //esama paveiksliuko padetis
                            //(rodo kelintas bmp tashkas yra ties (0,0) screen kampu).
static int max_x, max_y;    //max. leistinas scroolinimas (min. == 0)
static int screen_x, screen_y;  //efektyvus screen'o dydis

/** internal routine function
 Initiates the allegro library.
*/
void uzhkurt_alleg() {
    allegro_init();
    install_keyboard();
    install_mouse();
    install_timer();
    set_color_depth(24);    //TODO: patikrinti, ar chia nieko blogo del coloru
    return;
}
/** internal routine function
 Detection of the window close button has been pressed.
 I called by allegro, asynchroniously to the rest of \ref DyrkiView.cc.
 It only sets the global variable, and the main loop (\ref rodyt)
 notices that and breaks.
*/
void closeHook() {
    uzhdaryt = true;
    return;
}

/** internal routine function
 This function counts apropriate dimension of a screen.
*/
void getDims(int &w, int &h) {
    int wtemp = 0, htemp = 0;
    if (w < 100) w = 100;
    if (h < 100) h = 100;
    if (!get_desktop_resolution(&wtemp, &htemp)) {
        w = (w > (int)(wtemp*0.75)) ? int(wtemp*0.75) : w;
        h = (h > (int)(htemp*0.75)) ? int(htemp*0.75) : h;
    } else {
        w = 640;
        h = 480;
    }
}

/** internal routine function
 This function creates new allegro window.
 It sets some global variables, detects proper video mode,etc.
*/
void uzhkurt_GFX(int w, int h, const char *c) {
    set_window_close_hook(&closeHook);
    set_window_title((char *)c);
    getDims(w, h);
#ifdef DV_WIN32
    set_gfx_mode(GFX_GDI, w, h, w, h);
#else
	set_gfx_mode(GFX_XWINDOWS, w, h, w, h);
#endif
    screen_x = w; screen_y = h;
    show_mouse(screen);
}
/** internal routine function
 This function redraws the screen when user drags image.
*/
inline void perpiest(ALLEGRO_BITMAP *bmp, int x, int y) {
    if (max_x > 0) {    //jei galima scroolinti
        img_x -= x;     //kiek paslinkta su pele (busima img pradz. poz.)
        if (img_x > max_x) img_x = max_x; //jei nuslinkta per daug y shona
        if (img_x < 0) img_x = 0;         // --"--
    } else img_x = 0;   //negalima scroolinti, perpieshiame normaliai

    if (max_y > 0) {    //jei galima scroolinti
        img_y -= y;     //kiek paslinkta su pele (busima img pradz. poz.)
        if (img_y > max_y) img_y = max_y; //jei nuslinkta per daug y shona
        if (img_y < 0) img_y = 0;         // --"--
    } else img_y = 0;    //negalima scroolinti, perpieshiame normaliai

    scare_mouse();
    blit(bmp, screen, img_x, img_y, 0, 0, screen_x, screen_y);
    unscare_mouse();
}

/** A function for showing an image.
 The main function for displaying the image. It handles all the drawing routines.
 \param fn      Filename of an image file.
 \param title   Title of the window.
 \return        Always 0 (this should be changed...)
*/
int rodyt(const char *fn, const char *title) {
    ALLEGRO_BITMAP *bmp = NULL;
    RGB pal[256];
    bool toggle_mikrik = false;  //ar buvo pele  judinta su paspaustu mikriku
    int x = 0, y = 0;
    uzhdaryt = false;
    uzhkurt_alleg();
    bmp = load_bitmap(fn, pal);
    if (!bmp) {
        fprintf(stderr, "DyrkiView (child): Cannot open specified file: %s\n",fn);
        return 64;
    }
    uzhkurt_GFX(bmp->w, bmp->h, title);
    scare_mouse();
    blit(bmp, screen, 0, 0, 0, 0, bmp->w, bmp->h);
    unscare_mouse();
    img_x = 0; img_y = 0;
    max_x = (bmp->w - screen_x) > 0 ? bmp->w - screen_x : 0;
    max_y = (bmp->h - screen_y) > 0 ? bmp->h - screen_y : 0; 
//    while (!uzhdaryt && !keypressed()) {      //TODO: padaryti detektinima, kada nera mouso driverio
    while (!uzhdaryt)
    {
        poll_mouse();       //poll'iname zhiurka
        if (mouse_b & 2) break;
        if (!(mouse_b & 1)) {               //nepaspaustas mikrikas
            rest(100);
            toggle_mikrik = false;
        }
        else {
            if ((mouse_x != x) || (mouse_y != y)) { //zhiurka pajudinta
                if (toggle_mikrik) perpiest(bmp, mouse_x-x, mouse_y-y);
                x = mouse_x; y = mouse_y;  //^^ perpieshiame tik tada, kai buvo traukta
                toggle_mikrik = true;      //su paspaustu mikriku
            } //else rest(50);
        } //else'o 
    } //while'o
    destroy_bitmap(bmp);
    return 0;

}
