/** \file dv_fork.cc
    \author Robertas Dumpis
    \date 2002 12 13
    \version 0.9.12
    \brief A part of external viewer utility.
 This file is a part of DyrkiView, utility used for scaled image output
 on a separate window. DyrkiView is a separate executable, whick is optional
 to Dyrki project. It also compiles separately. DyrkiView currently compiles
 and runs only on linux and windows platforms.\n
 This part is responsible for creating an independant process of to display
 an image. It has separate implementations of dv_fork(), one for linux
 and other for windows, because these platforms uses completely different
 methods for creating a new process.\n
 \b Note:\n
 This file has nothing to do with dyrki project, except it is used by the project.
 */
 
#include <stdlib.h>
#include <stdio.h>
#ifdef DV_WIN32
#include <windows.h>
#include <strings.h>
#else
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#endif

//#include "mmgr.h"

#include "dv_fork.h"
#include "DyrkiView.h"


#ifdef DV_LINUX
/**The dv_fork() function for linux.
 This function uses fork() call to effectively duplicate current process.
 After call to fork() we have two processes: a parent (which must exit imeadetly)
 and a child, which has to do all the job.
 \param fn      Filename of an image file.
 \param title   Title of the window.
 \return        zero when child succesfully created (parent); \ref rodyt return value (child); nonzero on failure (parent);
 \b note:\n
 These parameters aren't used by this function, they are only bypassed to
 /ref rodyt
 */
int dv_fork(const char *fn, const char *title) {
    pid_t pp = -1;
    pp = fork();
    if (pp == 0) return rodyt(fn, title);       //I am a child ;))
    if (pp == -1) return 16;    //Error creating child process
    fprintf(stdout, "DyrkiView: parent returning successfuly.\n");   //I am a parent, lets return ;))
    return 0;                   
}
#endif

/*Gauruotas reikalas su windozemis
  CreateProcess() reikalauja velnishkai daug
  argumentu, msdn'e dokumentacija yra praktishkai _neyskaitoma_
  Taigi, buvo padarytas NULL hack'as ;)
  Ant WindowsXP dirba, ant kitur nzn.
  Microsoft'as rasho, kad turi dirbti ant visu windu...*/
//TODO: gali viskas pareiti su path'ais, atveju neuzhregistruota...
//Atvejai uzhregistruoti lokaliai, padetis ishtaisyta.

/**The dv_fork() function for windows.
  This one uses CreateProcess() to start another process of DyrkiView.
  The new process is given one extra argument, so \ref main.cc can
  detect that it's child and call the \ref DyrkiView.cc part to do
  the job. After execution of this function, program exits.
  \param fn      Filename of an image file.
  \param title   Title of the window.
  \return        zero on success, nonzero on failure.
  \b note:\n
  These parameters are used to form arguments for a new process.
*/
#ifdef DV_WIN32
int dv_fork(const char *fn, const char *title) {
    PROCESS_INFORMATION m_ProcessInformation;
    STARTUPINFO StartupInfo;
    char buff[128] = "\0";
    bool e_status = false;
    strcat(buff, "DyrkiView.exe ");
    strncat(buff, fn, 32);
    strcat(buff, " ");
    strncat(buff, title, 32);
    strcat(buff, " STA");
    fprintf(stdout, "Creating child process ( %s )", buff);    //trumpi parametrai...
    memset(&StartupInfo, 0, sizeof(StartupInfo));
    StartupInfo.cb = sizeof(StartupInfo);                   //CREATE_NEW_CONSOLE ?? DETACHED_PROCESS
    e_status = CreateProcess(NULL, buff, NULL, NULL, false, DETACHED_PROCESS, NULL, NULL, &StartupInfo, &m_ProcessInformation);
    if (e_status) {
        fprintf(stdout, "OK\n");
        return 0;
    } else {
        fprintf(stdout, "Failed\n");
        return 1;
    }
    if (!WaitForInputIdle(m_ProcessInformation.hProcess, 5000)) {
        fprintf(stdout, "Child process initialized successfuly.\n");
        return 0;
    } else {
        fprintf(stdout, "Child process failed at initialization.\n");
        return 2;
    }  
}
#endif
