/** \file main.cc
    \author Robertas Dumpis
    \date 2002 11 29
    \version 0.9.8
    \brief A part of external viewer utility.
 This file is a part of DyrkiView, utility used for scaled image output
 on a separate window. DyrkiView is a separate executable, whick is optional
 to Dyrki project. It also compiles separately. DyrkiView currently compiles
 and runs only on linux and windows platforms.\n
 This is the entry point of DyrkiView. It parses arguments and calls
 \ref dv_fork(). It also detects then the utility is called as a child process
 under windows platform (unix fork() call handles that differently) for details,
 look at \ref rodyt.
 \b Note:\n
 This file has nothing to do with dyrki project, except it is used by the project.
 */

#include <allegro.h>    //Naudojama rtfb allegro_hack.h
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef DV_WIN32
#include <strings.h>
#else 
#include <string.h>
#endif

//#include "mmgr.h"

#include "DyrkiView.h"
#include "dv_fork.h"

/** Function for checking program's arguments.*/
int parse_args(const char *fn, const char *title) {
    struct stat *buf = (struct stat*)malloc(sizeof(struct stat));
    if (stat(fn, buf)) {       //file doesn't exist or error
	free(buf);
        fprintf(stderr, "DyrkiView: Cannot find or open file specified: %s\n", fn);
        return 2;
    }    
    free(buf);
    if ((strlen(title) == 0) || strlen(title) > 31) {   //invalid title
        fprintf(stderr, "DyrkiView: Invalid window title specified\n");
        return 3;
    }   
    return 0;
}
/** An entry point... */
int main(int argc, char *argv[]) {
    int p_res = 0;
    fprintf(stdout, "DyrkiView ver. %s starting.\n", DV_VER);
    if (argc == 3) {
        p_res = parse_args(argv[1], argv[2]);
        if (p_res)  return p_res;           //bad arguments  
        fprintf(stderr, "Trying to fork()...\n");             
        return dv_fork(argv[1], argv[2]);   //args OK, continue
    }
#ifdef DV_WIN32
    if (argc == 4) return rodyt(argv[1], argv[2]);
#endif
    fprintf(stderr, "DyrkiView: Bad arguments\n");
    return 254;    
  }
END_OF_MAIN()
